@@ -89,21 +89,106 @@ def tiff2jp2():
 
     epilog = (
         "Normally you should at least provide the tilesize argument.  "
-        "tiff2jp2 will NOT automatically use the TIFF tile dimensions "
-        "(if tiled)."
+        "Even if the TIFF is tiled, tiff2jp2 will NOT automatically use the "
+        "TIFF tile dimensions."
     )
     kwargs = {
         'description': 'Convert TIFF to JPEG 2000.',
         'formatter_class': argparse.ArgumentDefaultsHelpFormatter,
-        'epilog': epilog
+        'epilog': epilog,
+        'add_help': False
     }
     parser = argparse.ArgumentParser(**kwargs)
 
-    help = 'Dimensions of JP2K tile.'
-    parser.add_argument(
-        '--tilesize', nargs=2, type=int, help=help, metavar=('h', 'w')
+    group1 = parser.add_argument_group(
+        'JP2K', 'Pass-through arguments to Jp2k.'
     )
 
+    help = 'Capture resolution parameters'
+    group1.add_argument(
+        '--capture-resolution', nargs=2, type=float, help=help,
+        metavar=('VRESC', 'HRESC')
+    )
+
+    help = 'Display resolution parameters'
+    group1.add_argument(
+        '--display-resolution', nargs=2, type=float, help=help,
+        metavar=('VRESD', 'HRESD')
+    )
+
+    help = (
+        'Compression ratio for successive layers.  You may specify more '
+        'than once to get multiple layers.'
+    )
+    group1.add_argument('--cratio', action='append', type=int, help=help)
+
+    help = (
+        'PSNR for successive layers.  You may specify more than once to get '
+        'multiple layers.'
+    )
+    group1.add_argument('--psnr', action='append', type=int, help=help)
+
+    help = 'Codeblock size.'
+    group1.add_argument(
+        '--codeblocksize', nargs=2, type=int, help=help,
+        metavar=('cblkh', 'cblkw')
+    )
+
+    help = 'Number of decomposition levels.'
+    group1.add_argument('--numres', type=int, help=help, default=6)
+
+    help = 'Progression order.'
+    choices = ['lrcp', 'rlcp', 'rpcl', 'prcl', 'cprl']
+    group1.add_argument('--prog', choices=choices, help=help, default='lrcp')
+
+    help = 'Use irreversible 9x7 transform.'
+    group1.add_argument('--irreversible', help=help, action='store_true')
+
+    help = 'Generate EPH markers.'
+    group1.add_argument('--eph', help=help, action='store_true')
+
+    help = 'Generate PLT markers.'
+    group1.add_argument('--plt', help=help, action='store_true')
+
+    help = 'Generate SOP markers.'
+    group1.add_argument('--sop', help=help, action='store_true')
+
+    group2 = parser.add_argument_group(
+        'TIFF', 'Arguments specific to conversion of TIFF imagery.'
+    )
+
+    help = 'Create Exif UUID box from TIFF metadata.'
+    group2.add_argument(
+        '--create-exif-uuid', help=help, action='store_true', default=True
+    )
+
+    help = (
+        'Extract XMLPacket tag value from TIFF IFD and store in XMP UUID box. '
+        'This will exclude the XMLPacket tag from the Exif UUID box.'
+    )
+    group2.add_argument('--create-xmp-uuid', help=help, action='store_true')
+
+    help = (
+        'Exclude TIFF tag(s) from EXIF UUID (if creating such a UUID).  '
+        'This option may be specified as tag numbers or names.'
+    )
+    group2.add_argument('--exclude-tags', help=help, nargs='*')
+
+    help = (
+        'Dimensions of JP2K tile.  If not provided, the JPEG2000 image will '
+        'be written as a single tile.'
+    )
+    group2.add_argument(
+        '--tilesize', nargs=2, type=int, help=help, metavar=('NROWS', 'NCOLS')
+    )
+
+    group2.add_argument('tifffile', help='Input TIFF file.')
+    group2.add_argument('jp2kfile', help='Output JPEG 2000 file.')
+
+    # These arguments are not specific to either group.
+    help = 'Show this help message and exit'
+    parser.add_argument('--help', '-h', action='help', help=help)
+
     help = (
         'Logging level, one of "critical", "error", "warning", "info", '
         'or "debug".'
@@ -113,53 +198,6 @@ def tiff2jp2():
         choices=['critical', 'error', 'warning', 'info', 'debug']
     )
 
-    help = (
-        'Compression ratio for successive layers.  You may specify more '
-        'than once to get multiple layers.'
-    )
-    parser.add_argument(
-        '--cratio', action='append', type=int, help=help,
-    )
-
-    help = (
-        'PSNR for successive layers.  You may specify more than once to get '
-        'multiple layers.'
-    )
-    parser.add_argument(
-        '--psnr', action='append', type=int, help=help,
-    )
-
-    help = 'Codeblock size.'
-    parser.add_argument(
-        '--codeblocksize', nargs=2, type=int, help=help,
-        metavar=('cblkh', 'cblkw')
-    )
-
-    help = 'Number of decomposition levels.'
-    parser.add_argument('--numres', type=int, help=help, default=6)
-
-    help = 'Progression order.'
-    choices = ['lrcp', 'rlcp', 'rpcl', 'prcl', 'cprl']
-    parser.add_argument('--prog', choices=choices, help=help, default='lrcp')
-
-    help = 'Use irreversible 9x7 transform.'
-    parser.add_argument('--irreversible', help=help, action='store_true')
-
-    help = 'Generate EPH markers.'
-    parser.add_argument('--eph', help=help, action='store_true')
-
-    help = 'Generate PLT markers.'
-    parser.add_argument('--plt', help=help, action='store_true')
-
-    help = 'Generate SOP markers.'
-    parser.add_argument('--sop', help=help, action='store_true')
-
-    help = 'Do not create UUID box for TIFF metadata.'
-    parser.add_argument('--nouuid', help=help, action='store_false')
-
-    parser.add_argument('tifffile')
-    parser.add_argument('jp2kfile')
-
     args = parser.parse_args()
 
     logging_level = getattr(logging, args.verbosity.upper())
@@ -167,10 +205,24 @@ def tiff2jp2():
     tiffpath = pathlib.Path(args.tifffile)
     jp2kpath = pathlib.Path(args.jp2kfile)
 
-    with Tiff2Jp2k(
-        tiffpath, jp2kpath, tilesize=args.tilesize, verbosity=logging_level,
-        cbsize=args.codeblocksize, cratios=args.cratio, numres=args.numres,
-        plt=args.plt, eph=args.eph, sop=args.sop, prog=args.prog,
-        irreversible=args.irreversible, psnr=args.psnr, create_uuid=args.nouuid
-    ) as j:
+    kwargs = {
+        'cbsize': args.codeblocksize,
+        'cratios': args.cratio,
+        'capture_resolution': args.capture_resolution,
+        'create_exif_uuid': args.create_exif_uuid,
+        'create_xmp_uuid': args.create_xmp_uuid,
+        'display_resolution': args.display_resolution,
+        'eph': args.eph,
+        'exclude_tags': args.exclude_tags,
+        'irreversible': args.irreversible,
+        'numres': args.numres,
+        'plt': args.plt,
+        'prog': args.prog,
+        'psnr': args.psnr,
+        'sop': args.sop,
+        'tilesize': args.tilesize,
+        'verbosity': logging_level,
+    }
+
+    with Tiff2Jp2k(tiffpath, jp2kpath, **kwargs) as j:
         j.run()
