@@ -4,6 +4,7 @@
 # Standard library imports
 import importlib.resources as ir
 import io
+import platform
 import shutil
 import struct
 import unittest
@@ -197,17 +198,65 @@ class TestSuite(fixtures.TestCommon):
         b.seek(0)
         return b.read()
 
+    def test__read_exif_uuid_missing_exif00_lead_in(self):
+        """
+        SCENARIO:  Parse a JpgTiffExif->Jp2 UUID that is missing the 'EXIF\0\0'
+        lead-in.
+
+        EXPECTED RESULT:  Should not error out.  Verify the UUID type.  Verify
+        the existance of one of the "Exif.Photo" tags.
+        """
+        box_data = ir.read_binary('tests.data', 'issue549.dat')
+        bf = io.BytesIO(box_data)
+        box = UUIDBox.parse(bf, 0, len(box_data))
+
+        actual = box.uuid
+        expected = uuid.UUID(bytes=b'JpgTiffExif->JP2')
+        self.assertEqual(actual, expected)
+
+        self.assertEqual(box.data['ExifTag']['ExifVersion'], (48, 50, 51, 50))
+
+    def test__read_malformed_exif_uuid(self):
+        """
+        SCENARIO:  Parse a JpgTiffExif->Jp2 UUID that is not only missing the
+        'EXIF\0\0' lead-in, but even the TIFF header is malformed.
+
+        EXPECTED RESULT:  RuntimeError
+        """
+        box_data = ir.read_binary('tests.data', 'issue549.dat')
+        bf = io.BytesIO(box_data[:16] + box_data[20:])
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            box = UUIDBox.parse(bf, 0, 37700)
+
+        actual = box.uuid
+        expected = uuid.UUID(bytes=b'JpgTiffExif->JP2')
+        self.assertEqual(actual, expected)
+
+    @unittest.skipIf(
+            platform.system().startswith('Windows'),
+            "Skipping on windows, see issue 560"
+    )
     def test__printing__geotiff_uuid__xml_sidecar(self):
         """
-        SCENARIO:  Print a geotiff UUID with XML sidecar file
+        SCENARIO:  Print a geotiff UUID with XML sidecar file.
 
-        EXPECTED RESULT:  Should not error out.
+        EXPECTED RESULT:  Should not error out.  There is a warning about GDAL
+        not being able to print the UUID data as expected.
         """
         box_data = ir.read_binary('tests.data', '0220000800_uuid.dat')
         bf = io.BytesIO(box_data)
         bf.seek(8)
         box = UUIDBox.parse(bf, 0, 703)
-        str(box)
+        with warnings.catch_warnings(record=True) as w:
+            str(box)
+
+        if fixtures._HAVE_GDAL:
+            self.assertEqual(len(w), 1)
+        else:
+            # If no gdal, there's no warning.  It's just an Exif UUID in
+            # that case.
+            self.assertEqual(len(w), 0)
 
     def test_append_xmp_uuid(self):
         """
@@ -312,29 +361,29 @@ class TestSuite(fixtures.TestCommon):
         expected = 'UTM Zone 16N NAD27"|Clarke, 1866 by Default| '
         self.assertEqual(box.data['GeoAsciiParams'], expected)
 
-    @unittest.skip('not sure why this was corrupt')
+    @unittest.skipIf(
+            platform.system().startswith('Windows'),
+            "Skipping on windows, see issue 560"
+    )
     def test_print_bad_geotiff(self):
         """
         SCENARIO:  A GeoTIFF UUID is corrupt.
 
-        EXPECTED RESULT:  The string representation should validate and clearly
-        state that the UUID box is corrupt.
+        EXPECTED RESULT:  No errors.  There is a warning issued when we try
+        to print the box.
         """
         with ir.path(data, 'issue398.dat') as path:
             with path.open('rb') as f:
                 f.seek(8)
-                with warnings.catch_warnings():
-                    # Ignore the warnings about invalid TIFF tags, we already
-                    # know that.
-                    warnings.simplefilter('ignore')
+                with warnings.catch_warnings(record=True) as w:
                     box = glymur.jp2box.UUIDBox.parse(f, 0, 380)
+                    str(box)
 
-        actual = str(box)
-        expected = ("UUID Box (uuid) @ (0, 380)\n"
-                    "    UUID:  "
-                    "b14bf8bd-083d-4b43-a5ae-8cd7d5a6ce03 (GeoTIFF)\n"
-                    "    UUID Data:  corrupt")
-        self.assertEqual(actual, expected)
+        if fixtures._HAVE_GDAL:
+            self.assertEqual(len(w), 1)
+        else:
+            # No warning issued if GDAL is not present.
+            self.assertEqual(len(w), 0)
 
 
 class TestSuiteHiRISE(fixtures.TestCommon):
