@@ -9,6 +9,7 @@ import importlib.resources as ir
 from io import BytesIO
 import os
 import pathlib
+import shutil
 import struct
 import tempfile
 import time
@@ -359,8 +360,13 @@ class TestJp2k(fixtures.TestCommon):
             with self.assertRaises(InvalidJp2kError):
                 Jp2k(path)
 
-    def test_read_from_a_file_that_does_not_exist(self):
-        """Should error out if reading from a file that does not exist"""
+    def test_file_does_not_exist(self):
+        """
+        Scenario:  The Jp2k construtor is passed a file that does not exist
+        and the intent is reading.
+
+        Expected Result:  FileNotFoundError
+        """
         # Verify that we error out appropriately if not given an existing file
         # at all.
         filename = 'this file does not actually exist on the file system.'
@@ -1120,6 +1126,112 @@ class TestJp2k_write(fixtures.MetadataBase):
                 else:
                     self.assertEqual(len(w), 1)
 
+    def test_capture_resolution(self):
+        """
+        SCENARIO:  The capture_resolution keyword is specified.
+
+        EXPECTED RESULT:  The cres box is created.
+        """
+        vresc, hresc = 0.1, 0.2
+        vresd, hresd = 0.3, 0.4
+        j = glymur.Jp2k(
+            self.temp_jp2_filename, data=self.jp2_data,
+            capture_resolution=[vresc, hresc],
+            display_resolution=[vresd, hresd],
+        )
+
+        self.assertEqual(j.box[2].box[2].box_id, 'res ')
+
+        self.assertEqual(j.box[2].box[2].box[0].box_id, 'resc')
+        self.assertEqual(j.box[2].box[2].box[0].vertical_resolution, vresc)
+        self.assertEqual(j.box[2].box[2].box[0].horizontal_resolution, hresc)
+
+        self.assertEqual(j.box[2].box[2].box[1].box_id, 'resd')
+        self.assertEqual(j.box[2].box[2].box[1].vertical_resolution, vresd)
+        self.assertEqual(j.box[2].box[2].box[1].horizontal_resolution, hresd)
+
+    def test_capture_resolution_when_j2k_specified(self):
+        """
+        Scenario:  Capture/Display resolution boxes are specified when the file
+        name indicates J2K.
+
+        Expected Result:  InvalidJp2kError
+        """
+
+        vresc, hresc = 0.1, 0.2
+        vresd, hresd = 0.3, 0.4
+        with self.assertRaises(InvalidJp2kError):
+            glymur.Jp2k(
+                self.temp_j2k_filename, data=self.jp2_data,
+                capture_resolution=[vresc, hresc],
+                display_resolution=[vresd, hresd],
+            )
+
+    def test_capture_resolution_when_not_writing(self):
+        """
+        Scenario:  Jp2k is invoked in a read-only situation but capture/display
+        resolution arguments are supplied.
+
+        Expected result:  RuntimeError
+        """
+        vresc, hresc = 0.1, 0.2
+        vresd, hresd = 0.3, 0.4
+
+        shutil.copyfile(self.jp2file, self.temp_jp2_filename)
+
+        with self.assertRaises(RuntimeError):
+            glymur.Jp2k(
+                self.temp_jp2_filename,
+                capture_resolution=[vresc, hresc],
+                display_resolution=[vresd, hresd],
+            )
+
+    def test_capture_resolution_supplied_but_not_display(self):
+        """
+        Scenario:  Writing a JP2 is intended, but only a capture resolution
+        box is specified, and not a display resolution box.
+
+        Expected Result:  No errors, the boxes are validated.
+        """
+        vresc, hresc = 0.1, 0.2
+
+        j = glymur.Jp2k(
+            self.temp_jp2_filename, data=self.jp2_data,
+            capture_resolution=[vresc, hresc],
+        )
+
+        self.assertEqual(j.box[2].box[2].box_id, 'res ')
+
+        self.assertEqual(j.box[2].box[2].box[0].box_id, 'resc')
+        self.assertEqual(j.box[2].box[2].box[0].vertical_resolution, vresc)
+        self.assertEqual(j.box[2].box[2].box[0].horizontal_resolution, hresc)
+
+        # there's just one child box
+        self.assertEqual(len(j.box[2].box[2].box), 1)
+
+    def test_display_resolution_supplied_but_not_capture(self):
+        """
+        Scenario:  Writing a JP2 is intended, but only a capture resolution
+        box is specified, and not a display resolution box.
+
+        Expected Result:  No errors, the boxes are validated.
+        """
+        vresd, hresd = 0.3, 0.4
+
+        j = glymur.Jp2k(
+            self.temp_jp2_filename, data=self.jp2_data,
+            display_resolution=[vresd, hresd],
+        )
+
+        self.assertEqual(j.box[2].box[2].box_id, 'res ')
+
+        self.assertEqual(j.box[2].box[2].box[0].box_id, 'resd')
+        self.assertEqual(j.box[2].box[2].box[0].vertical_resolution, vresd)
+        self.assertEqual(j.box[2].box[2].box[0].horizontal_resolution, hresd)
+
+        # there's just one child box
+        self.assertEqual(len(j.box[2].box[2].box), 1)
+
     def test_no_jp2c_box_in_outermost_jp2_list(self):
         """
         SCENARIO:  A JP2 file is encountered without a JP2C box in the outer-
@@ -1163,6 +1275,48 @@ class TestJp2k_write(fixtures.MetadataBase):
         with self.assertRaises(RuntimeError):
             Jp2k(self.temp_jp2_filename, **kwargs)
 
+    @unittest.skipIf(glymur.version.openjpeg_version < '2.5.0',
+                     "Requires as least v2.5.0")
+    def test_tlm_yes(self):
+        """
+        SCENARIO:  Use the tlm keyword.
+
+        EXPECTED RESULT:  A TLM segment is detected.
+        """
+        kwargs = {
+            'data': self.jp2_data,
+            'tlm': True
+        }
+        j = Jp2k(self.temp_jp2_filename, **kwargs)
+
+        codestream = j.get_codestream(header_only=False)
+
+        at_least_one_tlm_segment = any(
+            isinstance(seg, glymur.codestream.TLMsegment)
+            for seg in codestream.segment
+        )
+        self.assertTrue(at_least_one_tlm_segment)
+
+    def test_tlm_no(self):
+        """
+        SCENARIO:  Use the tlm keyword set to False
+
+        EXPECTED RESULT:  A TLM segment not detected.
+        """
+        kwargs = {
+            'data': self.jp2_data,
+            'tlm': False
+        }
+        j = Jp2k(self.temp_jp2_filename, **kwargs)
+
+        codestream = j.get_codestream(header_only=False)
+
+        at_least_one_tlm_segment = any(
+            isinstance(seg, glymur.codestream.TLMsegment)
+            for seg in codestream.segment
+        )
+        self.assertFalse(at_least_one_tlm_segment)
+
     @unittest.skipIf(
         not fixtures.HAVE_SCIKIT_IMAGE, fixtures.HAVE_SCIKIT_IMAGE_MSG
     )
