@@ -3,6 +3,7 @@ import importlib.resources as ir
 import logging
 import pathlib
 import shutil
+import struct
 import sys
 import tempfile
 import unittest
@@ -26,6 +27,125 @@ from glymur.lib import tiff as libtiff
 @unittest.skipIf(OPENJPEG_NOT_AVAILABLE, OPENJPEG_NOT_AVAILABLE_MSG)
 class TestSuite(fixtures.TestCommon):
 
+    @classmethod
+    def setup_exif(cls, path):
+        """
+        Create a simple TIFF file that is constructed to contain an EXIF IFD.
+        """
+
+        with path.open(mode='wb') as f:
+
+            w = 256
+            h = 256
+            rps = 64
+            header_length = 8
+
+            # write the header (8 bytes).  The IFD will follow the image data
+            # (256x256 bytes), so the offset to the IFD will be 8 + h * w.
+            main_ifd_offset = header_length + h * w
+            buffer = struct.pack('<BBHI', 73, 73, 42, main_ifd_offset)
+            f.write(buffer)
+
+            # write the image data, 4 64x256 strips of all zeros
+            strip = bytes([0] * rps * w)
+            f.write(strip)
+            f.write(strip)
+            f.write(strip)
+            f.write(strip)
+
+            # write an IFD with 11 tags
+            main_ifd_data_offset = main_ifd_offset + 2 + 11 * 12 + 4
+
+            buffer = struct.pack('<H', 11)
+            f.write(buffer)
+
+            # width and length and bitspersample
+            buffer = struct.pack('<HHII', 256, 4, 1, w)
+            f.write(buffer)
+            buffer = struct.pack('<HHII', 257, 4, 1, h)
+            f.write(buffer)
+            buffer = struct.pack('<HHII', 258, 4, 1, 8)
+            f.write(buffer)
+
+            # photometric
+            buffer = struct.pack('<HHII', 262, 4, 1, 1)
+            f.write(buffer)
+
+            # strip offsets
+            buffer = struct.pack('<HHII', 273, 4, 4, main_ifd_data_offset)
+            f.write(buffer)
+
+            # spp
+            buffer = struct.pack('<HHII', 277, 4, 1, 1)
+            f.write(buffer)
+
+            # rps
+            buffer = struct.pack('<HHII', 278, 4, 1, 64)
+            f.write(buffer)
+
+            # strip byte counts
+            buffer = struct.pack('<HHII', 279, 4, 4, main_ifd_data_offset + 16)
+            f.write(buffer)
+
+            # pagenumber
+            buffer = struct.pack('<HHIHH', 297, 3, 2, 1, 0)
+            f.write(buffer)
+
+            # XMP
+            with ir.path('tests.data', 'issue555.xmp') as xmp_path:
+                with xmp_path.open() as f2:
+                    xmp = f2.read()
+                    xmp = xmp + '\0'
+            buffer = struct.pack(
+                '<HHII', 700, 1, len(xmp), main_ifd_data_offset + 32
+            )
+            f.write(buffer)
+
+            # exif tag
+            exif_ifd_offset = main_ifd_data_offset + 32 + len(xmp)
+            buffer = struct.pack('<HHII', 34665, 4, 1, exif_ifd_offset)
+            f.write(buffer)
+
+            # terminate the IFD
+            buffer = struct.pack('<I', 0)
+            f.write(buffer)
+
+            # write the strip offsets here
+            buffer = struct.pack(
+                '<IIII', 8, 8 + rps*w, 8 + 2*rps*w, 8 + 3*rps*w
+            )
+            f.write(buffer)
+
+            # write the strip byte counts
+            buffer = struct.pack('<IIII', rps*w, rps*w, rps*w, rps*w)
+            f.write(buffer)
+
+            # write the XMP data
+            f.write(xmp.encode('utf-8'))
+
+            # write a minimal Exif IFD
+            buffer = struct.pack('<H', 2)
+            f.write(buffer)
+
+            # exposure program
+            buffer = struct.pack('<HHIHH', 34850, 3, 1, 2, 0)
+            f.write(buffer)
+
+            # lens model
+            data_location = exif_ifd_offset + 2 + 2*12 + 4
+            buffer = struct.pack('<HHII', 42036, 2, 6, data_location)
+            f.write(buffer)
+
+            # terminate the IFD
+            buffer = struct.pack('<I', 0)
+            f.write(buffer)
+
+            data = 'Canon\0'.encode('utf-8')
+            buffer = struct.pack('<BBBBBB', *data)
+            f.write(buffer)
+
+        cls.exif_tiff = path
+
     @classmethod
     def setup_minisblack_spp1(cls, path):
         """
@@ -38,7 +158,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(path, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.MINISBLACK)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
         libtiff.setField(fp, 'TileLength', th)
@@ -89,7 +209,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(path, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.MINISBLACK)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', h)
         libtiff.setField(fp, 'ImageWidth', w)
         libtiff.setField(fp, 'TileLength', th)
@@ -121,7 +241,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(path, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.MINISBLACK)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
         libtiff.setField(fp, 'TileLength', th)
@@ -159,7 +279,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(path, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.MINISBLACK)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
         libtiff.setField(fp, 'RowsPerStrip', rps)
@@ -191,7 +311,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(path, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.MINISBLACK)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
         libtiff.setField(fp, 'RowsPerStrip', rps)
@@ -223,7 +343,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(path, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.RGB)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
         libtiff.setField(fp, 'TileLength', th)
@@ -263,6 +383,53 @@ class TestSuite(fixtures.TestCommon):
         cls.astronaut_uint16_data = actual_data
         cls.astronaut_uint16_filename = path
 
+    @classmethod
+    def setup_ycbcr_striped_jpeg(cls, path):
+        """
+        SCENARIO:  create a simple color 2x1 stripped image
+        """
+        data = fixtures.skimage.data.astronaut()
+        h, w, z = data.shape
+        rps = h // 2
+
+        fp = libtiff.open(path, mode='w')
+
+        libtiff.setField(fp, 'Photometric', libtiff.Photometric.YCBCR)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.JPEG)
+
+        l, w = data.shape[:2]
+        libtiff.setField(fp, 'ImageLength', l)
+        libtiff.setField(fp, 'ImageWidth', w)
+        libtiff.setField(fp, 'RowsPerStrip', rps)
+
+        libtiff.setField(fp, 'BitsPerSample', 8)
+        libtiff.setField(fp, 'SamplesPerPixel', 3)
+        libtiff.setField(fp, 'PlanarConfig', libtiff.PlanarConfig.CONTIG)
+        libtiff.setField(fp, 'JPEGColorMode', libtiff.PlanarConfig.CONTIG)
+        libtiff.setField(fp, 'JPEGQuality', 100)
+
+        libtiff.writeEncodedStrip(fp, 0, data[:rps, :, :])
+        libtiff.writeEncodedStrip(fp, 1, data[rps:rps * 2, :, :])
+
+        libtiff.close(fp)
+
+        # now read it back
+        fp = libtiff.open(path)
+
+        strip = np.zeros((rps, w, 4), dtype=np.uint8)
+        actual_data = np.zeros((h, w, 3), dtype=np.uint8)
+
+        libtiff.readRGBAStrip(fp, 0, strip)
+        actual_data[:rps, :, :] = strip[::-1, :, :3]
+
+        libtiff.readRGBAStrip(fp, rps, strip)
+        actual_data[rps:rps * 2, :, :] = strip[::-1, :, :3]
+
+        libtiff.close(fp)
+
+        cls.astronaut_ycbcr_striped_jpeg_data = actual_data
+        cls.astronaut_ycbcr_striped_jpeg_tif = path
+
     @classmethod
     def setup_ycbcr_jpeg(cls, path):
         """
@@ -328,7 +495,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(path, mode='w8')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.RGB)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
         libtiff.setField(fp, 'TileLength', th)
@@ -379,7 +546,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(path, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.RGB)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
         libtiff.setField(fp, 'TileLength', th)
@@ -423,6 +590,8 @@ class TestSuite(fixtures.TestCommon):
         cls.test_tiff_dir = tempfile.mkdtemp()
         cls.test_tiff_path = pathlib.Path(cls.test_tiff_dir)
 
+        cls.setup_exif(cls.test_tiff_path / 'exif.tif')
+
         cls.setup_minisblack_spp1(cls.test_tiff_path / 'moon.tif')
 
         cls.setup_minisblack_3x3(cls.test_tiff_path / 'minisblack_3x3.tif')
@@ -442,12 +611,53 @@ class TestSuite(fixtures.TestCommon):
             cls.test_tiff_path / 'astronaut_ycbcr_jpeg_tiled.tif'
         )
 
+        cls.setup_ycbcr_striped_jpeg(
+            cls.test_tiff_path / 'astronaut_ycbcr_striped_jpeg.tif'
+        )
+
         cls.setup_rgb_uint16(cls.test_tiff_path / 'astronaut_uint16.tif')
 
     @classmethod
     def tearDownClass(cls):
         shutil.rmtree(cls.test_tiff_dir)
 
+    def test_exclude_tags_camelcase(self):
+        """
+        Scenario:  Convert TIFF to JP2, but exclude the StripByteCounts and
+        StripOffsets tags.  Supply the argments as camel-case.
+
+        Expected Result:  No warnings, no errors.  The Exif LensModel tag is
+        recoverable from the UUIDbox.
+        """
+        with Tiff2Jp2k(
+            self.exif_tiff, self.temp_jp2_filename,
+            exclude_tags=['StripOffsets', 'StripByteCounts']
+        ) as p:
+            p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        tags = j.box[-1].data
+        self.assertNotIn('StripByteCounts', tags)
+        self.assertNotIn('StripOffsets', tags)
+
+    def test_exif(self):
+        """
+        Scenario:  Convert TIFF with Exif IFD to JP2
+
+        Expected Result:  No warnings, no errors.  The Exif LensModel tag is
+        recoverable from the UUIDbox.
+        """
+        with Tiff2Jp2k(self.exif_tiff, self.temp_jp2_filename) as p:
+            with warnings.catch_warnings(record=True) as w:
+                p.run()
+                self.assertEqual(len(w), 0)
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        tags = j.box[-1].data
+        self.assertEqual(tags['ExifTag']['LensModel'], 'Canon')
+
     def test_smoke(self):
         """
         SCENARIO:  Convert TIFF file to JP2
@@ -508,11 +718,73 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(j.box[-1].data['ImageWidth'], 512)
         self.assertEqual(j.box[-1].data['ImageLength'], 512)
 
+    def test_smoke_rgba(self):
+        """
+        SCENARIO:  Convert RGCA TIFF file to JP2
+
+        EXPECTED RESULT:  data matches, number of resolution is the default.
+        There should be just one layer.  The number of resolutions should be
+        the default (5).  There are not PLT segments.  There are no EPH
+        markers.  There are no SOP markers.  The progression order is LRCP.
+        The irreversible transform will NOT be used.  PSNR cannot be tested
+        if it is not applied.
+
+        There is a UUID box appended at the end containing the metadata.
+        """
+        with Tiff2Jp2k(
+            self.astronaut_ycbcr_striped_jpeg_tif, self.temp_jp2_filename,
+            tilesize=[256, 256]
+        ) as j:
+            j.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        actual = j[:]
+        self.assertEqual(actual.shape, (512, 512, 3))
+
+        c = j.get_codestream(header_only=False)
+
+        actual = c.segment[2].code_block_size
+        expected = (64, 64)
+        self.assertEqual(actual, expected)
+
+        self.assertEqual(c.segment[2].layers, 1)
+        self.assertEqual(c.segment[2].num_res, 5)
+
+        at_least_one_eph = any(
+            isinstance(seg, glymur.codestream.EPHsegment)
+            for seg in c.segment
+        )
+        self.assertFalse(at_least_one_eph)
+
+        at_least_one_plt = any(
+            isinstance(seg, glymur.codestream.PLTsegment)
+            for seg in c.segment
+        )
+        self.assertFalse(at_least_one_plt)
+
+        at_least_one_sop = any(
+            isinstance(seg, glymur.codestream.SOPsegment)
+            for seg in c.segment
+        )
+        self.assertFalse(at_least_one_sop)
+
+        self.assertEqual(c.segment[2].prog_order, glymur.core.LRCP)
+
+        self.assertEqual(
+            c.segment[2].xform, glymur.core.WAVELET_XFORM_5X3_REVERSIBLE
+        )
+
+        self.assertEqual(j.box[-1].box_id, 'uuid')
+        self.assertEqual(j.box[-1].data['ImageWidth'], 512)
+        self.assertEqual(j.box[-1].data['ImageLength'], 512)
+
     def test_geotiff(self):
         """
-        SCENARIO:  Convert GEOTIFF file to JP2
+        SCENARIO:  Convert a one-component GEOTIFF file to JP2
 
-        EXPECTED RESULT:  there is a geotiff UUID.
+        EXPECTED RESULT:  there is a geotiff UUID.  The JP2 file has only one
+        component.
         """
         with warnings.catch_warnings():
             warnings.simplefilter('ignore')
@@ -526,6 +798,7 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(
             j.box[-1].uuid, UUID('b14bf8bd-083d-4b43-a5ae-8cd7d5a6ce03')
         )
+        self.assertEqual(j.box[2].box[0].num_components, 1)
 
     def test_no_uuid(self):
         """
@@ -536,7 +809,7 @@ class TestSuite(fixtures.TestCommon):
         """
         with Tiff2Jp2k(
             self.astronaut_ycbcr_jpeg_tif, self.temp_jp2_filename,
-            create_uuid=False
+            create_exif_uuid=False
         ) as j:
             j.run()
 
@@ -800,7 +1073,7 @@ class TestSuite(fixtures.TestCommon):
         """
         with Tiff2Jp2k(
             self.minisblack_3strip_partial_last_strip, self.temp_jp2_filename,
-            tilesize=(240, 240)
+            tilesize=(240, 240), verbose='DEBUG'
         ) as j:
             j.run()
 
@@ -829,7 +1102,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(self.temp_tiff_filename, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.MINISBLACK)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'SampleFormat', libtiff.SampleFormat.UINT)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
@@ -863,7 +1136,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(self.temp_tiff_filename, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.MINISBLACK)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'SampleFormat', libtiff.SampleFormat.IEEEFP)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
@@ -1215,6 +1488,27 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(c.segment[1].xtsiz, 256)
         self.assertEqual(c.segment[1].ytsiz, 256)
 
+    def test_commandline_tiff2jp2_exclude_tags_numeric(self):
+        """
+        Scenario:  patch sys such that we can run the command line tiff2jp2
+        script.  Exclude TileByteCounts and TileByteOffsets, but provide those
+        tags as numeric values.
+
+        Expected Results:  Same as test_astronaut.
+        """
+        sys.argv = [
+            '', str(self.astronaut_tif), str(self.temp_jp2_filename),
+            '--tilesize', '256', '256',
+            '--exclude-tags', '324', '325'
+        ]
+        command_line.tiff2jp2()
+
+        jp2 = Jp2k(self.temp_jp2_filename)
+        tags = jp2.box[-1].data
+
+        self.assertNotIn('TileByteCounts', tags)
+        self.assertNotIn('TileOffsets', tags)
+
     def test_cmyk(self):
         """
         Scenario:  CMYK (or separated) is not a supported colorspace.
@@ -1232,7 +1526,7 @@ class TestSuite(fixtures.TestCommon):
         fp = libtiff.open(self.temp_tiff_filename, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.SEPARATED)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
         libtiff.setField(fp, 'RowsPerStrip', rps)
@@ -1252,6 +1546,27 @@ class TestSuite(fixtures.TestCommon):
                 with self.assertRaises(RuntimeError):
                     j.run()
 
+    def test_commandline_tiff2jp2_exclude_tags(self):
+        """
+        Scenario:  patch sys such that we can run the command line tiff2jp2
+        script.  Exclude TileByteCounts and TileByteOffsets
+
+        Expected Results:  TileByteCounts and TileOffsets are not in the EXIF
+        UUID.
+        """
+        sys.argv = [
+            '', str(self.astronaut_tif), str(self.temp_jp2_filename),
+            '--tilesize', '256', '256',
+            '--exclude-tags', 'tilebytecounts', 'tileoffsets'
+        ]
+        command_line.tiff2jp2()
+
+        jp2 = Jp2k(self.temp_jp2_filename)
+        tags = jp2.box[-1].data
+
+        self.assertNotIn('TileByteCounts', tags)
+        self.assertNotIn('TileOffsets', tags)
+
 
 class TestSuiteNoScikitImage(fixtures.TestCommon):
 
@@ -1263,6 +1578,127 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
 
         cls.setup_rgb_evenly_stripped(cls.test_tiff_path / 'goodstuff.tif')
 
+        cls.setup_exif(cls.test_tiff_path / 'exif.tif')
+
+    @classmethod
+    def setup_exif(cls, path):
+        """
+        Create a simple TIFF file that is constructed to contain an EXIF IFD.
+        """
+
+        with path.open(mode='wb') as f:
+
+            w = 256
+            h = 256
+            rps = 64
+            header_length = 8
+
+            # write the header (8 bytes).  The IFD will follow the image data
+            # (256x256 bytes), so the offset to the IFD will be 8 + h * w.
+            main_ifd_offset = header_length + h * w
+            buffer = struct.pack('<BBHI', 73, 73, 42, main_ifd_offset)
+            f.write(buffer)
+
+            # write the image data, 4 64x256 strips of all zeros
+            strip = bytes([0] * rps * w)
+            f.write(strip)
+            f.write(strip)
+            f.write(strip)
+            f.write(strip)
+
+            # write an IFD with 11 tags
+            main_ifd_data_offset = main_ifd_offset + 2 + 11 * 12 + 4
+
+            buffer = struct.pack('<H', 11)
+            f.write(buffer)
+
+            # width and length and bitspersample
+            buffer = struct.pack('<HHII', 256, 4, 1, w)
+            f.write(buffer)
+            buffer = struct.pack('<HHII', 257, 4, 1, h)
+            f.write(buffer)
+            buffer = struct.pack('<HHII', 258, 4, 1, 8)
+            f.write(buffer)
+
+            # photometric
+            buffer = struct.pack('<HHII', 262, 4, 1, 1)
+            f.write(buffer)
+
+            # strip offsets
+            buffer = struct.pack('<HHII', 273, 4, 4, main_ifd_data_offset)
+            f.write(buffer)
+
+            # spp
+            buffer = struct.pack('<HHII', 277, 4, 1, 1)
+            f.write(buffer)
+
+            # rps
+            buffer = struct.pack('<HHII', 278, 4, 1, 64)
+            f.write(buffer)
+
+            # strip byte counts
+            buffer = struct.pack('<HHII', 279, 4, 4, main_ifd_data_offset + 16)
+            f.write(buffer)
+
+            # pagenumber
+            buffer = struct.pack('<HHIHH', 297, 3, 2, 1, 0)
+            f.write(buffer)
+
+            # XMP
+            with ir.path('tests.data', 'issue555.xmp') as xmp_path:
+                with xmp_path.open() as f2:
+                    xmp = f2.read()
+                    xmp = xmp + '\0'
+            buffer = struct.pack(
+                '<HHII', 700, 1, len(xmp), main_ifd_data_offset + 32
+            )
+            f.write(buffer)
+
+            # exif tag
+            exif_ifd_offset = main_ifd_data_offset + 32 + len(xmp)
+            buffer = struct.pack('<HHII', 34665, 4, 1, exif_ifd_offset)
+            f.write(buffer)
+
+            # terminate the IFD
+            buffer = struct.pack('<I', 0)
+            f.write(buffer)
+
+            # write the strip offsets here
+            buffer = struct.pack(
+                '<IIII', 8, 8 + rps*w, 8 + 2*rps*w, 8 + 3*rps*w
+            )
+            f.write(buffer)
+
+            # write the strip byte counts
+            buffer = struct.pack('<IIII', rps*w, rps*w, rps*w, rps*w)
+            f.write(buffer)
+
+            # write the XMP data
+            f.write(xmp.encode('utf-8'))
+
+            # write a minimal Exif IFD
+            buffer = struct.pack('<H', 2)
+            f.write(buffer)
+
+            # exposure program
+            buffer = struct.pack('<HHIHH', 34850, 3, 1, 2, 0)
+            f.write(buffer)
+
+            # lens model
+            data_location = exif_ifd_offset + 2 + 2*12 + 4
+            buffer = struct.pack('<HHII', 42036, 2, 6, data_location)
+            f.write(buffer)
+
+            # terminate the IFD
+            buffer = struct.pack('<I', 0)
+            f.write(buffer)
+
+            data = 'Canon\0'.encode('utf-8')
+            buffer = struct.pack('<BBBBBB', *data)
+            f.write(buffer)
+
+        cls.exif_tiff = path
+
     @classmethod
     def setup_rgb_evenly_stripped(cls, path):
         """
@@ -1276,7 +1712,7 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
         fp = libtiff.open(path, mode='w')
 
         libtiff.setField(fp, 'Photometric', libtiff.Photometric.RGB)
-        libtiff.setField(fp, 'Compression', libtiff.Compression.DEFLATE)
+        libtiff.setField(fp, 'Compression', libtiff.Compression.ADOBE_DEFLATE)
         libtiff.setField(fp, 'ImageLength', data.shape[0])
         libtiff.setField(fp, 'ImageWidth', data.shape[1])
         libtiff.setField(fp, 'RowsPerStrip', rps)
@@ -1298,7 +1734,8 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
         """
         Scenario:  input TIFF is organized by strips and logging is turned on.
 
-        Expected result:  there are 104 log messages, one for each tile
+        Expected result:  there are 104 log messages. These messages come from
+        the tiles (a 13x8 grid of tiles).
         """
         with Tiff2Jp2k(
             self.goodstuff_path, self.temp_jp2_filename, tilesize=(64, 64),
@@ -1355,3 +1792,257 @@ class TestSuiteNoScikitImage(fixtures.TestCommon):
         self.assertEqual(c.segment[1].ysiz, 800)
         self.assertEqual(c.segment[1].xtsiz, 75)
         self.assertEqual(c.segment[1].ytsiz, 75)
+
+    def test_exclude_tags(self):
+        """
+        Scenario:  Convert TIFF to JP2, but exclude the StripByteCounts and
+        StripOffsets tags.
+
+        Expected Result:  No warnings, no errors.  The Exif LensModel tag is
+        recoverable from the UUIDbox.
+        """
+        with Tiff2Jp2k(
+            self.exif_tiff, self.temp_jp2_filename,
+            exclude_tags=[273, 'stripbytecounts']
+        ) as p:
+            p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        tags = j.box[-1].data
+        self.assertNotIn('StripByteCounts', tags)
+        self.assertNotIn('StripOffsets', tags)
+
+        str(j.box[-1])
+
+    def test_exclude_tags_but_specify_a_bad_tag(self):
+        """
+        Scenario:  Convert TIFF to JP2, but exclude the StripByteCounts and
+        StripOffsets tags.  In addition, specify a tag that is not recognized.
+
+        Expected Result:  The results should be the same as the previous
+        test except that a warning is issued due to the bad tag.
+        """
+        with self.assertWarns(UserWarning):
+            with Tiff2Jp2k(
+                self.exif_tiff, self.temp_jp2_filename,
+                exclude_tags=[273, 'stripbytecounts', 'gdalstuff']
+            ) as p:
+                p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        tags = j.box[-1].data
+        self.assertNotIn('StripByteCounts', tags)
+        self.assertNotIn('StripOffsets', tags)
+
+    def test_exclude_tags_camelcase(self):
+        """
+        Scenario:  Convert TIFF to JP2, but exclude the StripByteCounts and
+        StripOffsets tags.  Supply the argments as camel-case.
+
+        Expected Result:  No warnings, no errors.  The Exif LensModel tag is
+        recoverable from the UUIDbox.
+        """
+        with Tiff2Jp2k(
+            self.exif_tiff, self.temp_jp2_filename,
+            exclude_tags=['StripOffsets', 'StripByteCounts']
+        ) as p:
+            p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        tags = j.box[-1].data
+        self.assertNotIn('StripByteCounts', tags)
+        self.assertNotIn('StripOffsets', tags)
+
+    def test_exif(self):
+        """
+        Scenario:  Convert TIFF with Exif IFD to JP2
+
+        Expected Result:  No warnings, no errors.  The Exif LensModel tag is
+        recoverable from the UUIDbox.
+        """
+        with Tiff2Jp2k(self.exif_tiff, self.temp_jp2_filename) as p:
+            with warnings.catch_warnings(record=True) as w:
+                p.run()
+                self.assertEqual(len(w), 0)
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        tags = j.box[-1].data
+        self.assertEqual(tags['ExifTag']['LensModel'], 'Canon')
+
+        str(j.box[-1])
+
+    def test_xmp(self):
+        """
+        Scenario:  Convert TIFF with Exif IFD to JP2.  The main IFD has an
+        XML Packet tag (700).  Supply the 'xmp_uuid' keyword.
+
+        Expected Result:  The XMLPacket tag is removed from the main IFD.
+        An Exif UUID is appended to the end of the JP2 file, and then an XMP
+        UUID is appended.
+        """
+        with Tiff2Jp2k(
+            self.exif_tiff, self.temp_jp2_filename, create_xmp_uuid=True
+        ) as p:
+            p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # first we find the Exif UUID, then the XMP UUID.  The Exif UUID
+        # data should not have the XMLPacket tag.
+        actual = j.box[-2].uuid
+        expected = UUID(bytes=b'JpgTiffExif->JP2')
+        self.assertEqual(actual, expected)
+        self.assertNotIn('XMLPacket', j.box[-2].data)
+
+        actual = j.box[-1].uuid
+        expected = UUID('be7acfcb-97a9-42e8-9c71-999491e3afac')
+        self.assertEqual(actual, expected)
+        self.assertEqual(
+            j.box[-1].data.getroot().values(), ['Public XMP Toolkit Core 3.5']
+        )
+
+    def test_commandline__capture_display_resolution__no_tilesize(self):
+        """
+        Scenario:  patch sys such that we can run the command
+        line tiff2jp2 script.  Supply the --capture-resolution and
+        --display-resolution arguments.
+
+        Expected Result:  The last box is a ResolutionBox.
+        """
+        vresc, hresc = 0.1, 0.2
+        vresd, hresd = 0.3, 0.4
+
+        sys.argv = [
+            '', str(self.exif_tiff), str(self.temp_jp2_filename),
+            '--capture-resolution', str(vresc), str(hresc),
+            '--display-resolution', str(vresd), str(hresd),
+        ]
+        command_line.tiff2jp2()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # the resolution superbox is appended in the jp2 header box.
+        # the exit uuid comes later
+        self.assertEqual(j.box[-1].box_id, 'uuid')
+
+        self.assertEqual(j.box[2].box[2].box_id, 'res ')
+
+        self.assertEqual(j.box[2].box[2].box[0].box_id, 'resc')
+        self.assertEqual(j.box[2].box[2].box[0].vertical_resolution, vresc)
+        self.assertEqual(j.box[2].box[2].box[0].horizontal_resolution, hresc)
+
+        self.assertEqual(j.box[2].box[2].box[1].box_id, 'resd')
+        self.assertEqual(j.box[2].box[2].box[1].vertical_resolution, vresd)
+        self.assertEqual(j.box[2].box[2].box[1].horizontal_resolution, hresd)
+
+    def test_commandline__capture_display_resolution__tilesize(self):
+        """
+        Scenario:  patch sys such that we can run the command line
+        tiff2jp2 script.  Supply the --tilesize, --capture-resolution
+        and --display-resolution arguments.
+
+        Expected Result:  The last box is a ResolutionBox.
+        """
+        vresc, hresc = 0.1, 0.2
+        vresd, hresd = 0.3, 0.4
+
+        sys.argv = [
+            '', str(self.exif_tiff), str(self.temp_jp2_filename),
+            '--tilesize', '64', '64',
+            '--capture-resolution', str(vresc), str(hresc),
+            '--display-resolution', str(vresd), str(hresd),
+        ]
+        command_line.tiff2jp2()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # the resolution superbox is appended in the jp2 header box.
+        # the exit uuid comes later
+        self.assertEqual(j.box[-1].box_id, 'uuid')
+
+        self.assertEqual(j.box[2].box[2].box_id, 'res ')
+
+        self.assertEqual(j.box[2].box[2].box[0].box_id, 'resc')
+        self.assertEqual(j.box[2].box[2].box[0].vertical_resolution, vresc)
+        self.assertEqual(j.box[2].box[2].box[0].horizontal_resolution, hresc)
+
+        self.assertEqual(j.box[2].box[2].box[1].box_id, 'resd')
+        self.assertEqual(j.box[2].box[2].box[1].vertical_resolution, vresd)
+        self.assertEqual(j.box[2].box[2].box[1].horizontal_resolution, hresd)
+
+    def test_commandline_tiff2jp2_xmp_uuid(self):
+        """
+        Scenario:  patch sys such that we can run the command line tiff2jp2
+        script.  Use the --create-xmp-uuid option.
+
+        Expected Result:  The XMLPacket tag is removed from the main IFD.
+        An Exif UUID is appended to the end of the JP2 file, and then an XMP
+        UUID is appended.
+        """
+        sys.argv = [
+            '', str(self.exif_tiff), str(self.temp_jp2_filename),
+            '--tilesize', '64', '64',
+            '--create-xmp-uuid'
+        ]
+        command_line.tiff2jp2()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        # first we find the Exif UUID, then the XMP UUID.  The Exif UUID
+        # data should not have the XMLPacket tag.
+        actual = j.box[-2].uuid
+        expected = UUID(bytes=b'JpgTiffExif->JP2')
+        self.assertEqual(actual, expected)
+        self.assertNotIn('XMLPacket', j.box[-2].data)
+
+        actual = j.box[-1].uuid
+        expected = UUID('be7acfcb-97a9-42e8-9c71-999491e3afac')
+        self.assertEqual(actual, expected)
+        self.assertEqual(
+            j.box[-1].data.getroot().values(), ['Public XMP Toolkit Core 3.5']
+        )
+
+    def test_one_component_no_tilesize(self):
+        """
+        Scenario:  The jp2 tilesize is the same as the image size.
+
+        Expected Result:  No errors.
+        """
+        with Tiff2Jp2k(
+            self.exif_tiff, self.temp_jp2_filename,
+        ) as p:
+            p.run()
+
+        j = Jp2k(self.temp_jp2_filename)
+        self.assertEqual(j.box[2].box[0].num_components, 1)
+
+    @unittest.skip('segfaulting')
+    def test_one_component_tilesize(self):
+        """
+        Scenario:  The jp2 tilesize is the same as the image size,
+        and the tilesize is specified.
+
+        Expected Result:  No errors.
+        """
+        with Tiff2Jp2k(
+            self.exif_tiff, self.temp_jp2_filename, tilesize=[256, 256]
+        ) as p:
+            p.run()
+
+        Jp2k(self.temp_jp2_filename)
+
+    def test_not_a_tiff(self):
+        """
+        Scenario:  The input "TIFF" is not actually a TIFF.  This used to
+        segfault.
+
+        Expected Result:  no segfault
+        """
+        with self.assertRaises(RuntimeError):
+            with ir.path('tests.data', 'simple_rdf.txt') as path:
+                with Tiff2Jp2k(path, self.temp_jp2_filename):
+                    pass
